#!/usr/local/bin/perl

package DBAccess;

use lib "/usr/local/perl/lib";
use XML::DOM;

require("Config.pl");
use Config;
@ISA = qw(Config);

my $parser;
my $doc;

my $codebase;
my $imagebase;
my $AbsoluteFileName;
my $AbsoluteFolderName;
my $ImageMagickBin;
my $db_foldername;
my $db_filename;

my @allowedtags = ("P", "B", "I", "A");

sub new {

#Para: "Config FileName", "DB FolderName from PartyTime root", "DB FileName"

  my $class = shift;
  my $config_filename = shift;
  $db_foldername = shift;
  $db_filename = shift;

  my $self = Config->new ("$config_filename");

  my $cgiPath = $self->{cgiPath};
  my $cgiURL = $self->{cgiURL};
  my $docrootPath = $self->{docrootPath};
  my $docrootURL = $self->{docrootURL};
  my $partytimeCGIDir = $self->{partytimeCGIDir};
  my $partytimeDOCDir = $self->{partytimeDOCDir};

  $ImageMagickBin = $self->{ImageMagickBin};

  $AbsoluteFileName="$docrootPath/$partytimeDOCDir/$db_foldername/$db_filename";
  $AbsoluteFolderName="$docrootPath/$partytimeDOCDir/$db_foldername";
  $codebase = "$cgiURL/$partytimeCGIDir";
  $imagebase = "$docrootURL/$partytimeDOCDir/$db_foldername";

  $parser = new XML::DOM::Parser;
  if ( -e "$AbsoluteFileName" ) {
    $doc = $parser->parsefile ("$AbsoluteFileName");
  } else {
    # new doc and default settings
    $doc = new XML::DOM::Document;
    my $albumdb = $doc->createElement ("AlbumDB");

    my $ImageBase = $doc->createElement ("ImageBase");
    $ImageBase->setAttribute ("url", "");
    $albumdb->appendChild ($ImageBase);

    my $AllowComment = $doc->createElement ("AllowComment");
    $AllowComment->setAttribute ("status", "yes");
    $albumdb->appendChild ($AllowComment);

    my $AllowUserUpload = $doc->createElement ("AllowUserUpload");
    $AllowUserUpload->setAttribute ("status", "yes");
    $albumdb->appendChild ($AllowUserUpload);

    $doc -> appendChild ( $albumdb );
  }

  bless($self);
  return $self;
}

sub newLocal {

#Para: "Config filename, Folder absolute name, DB FileName"

  my $class = shift;
  my $config_filename = shift;
  my $db_foldername = shift;
  my $db_filename = shift;

  my $self = Config->new ("$config_filename");

  my $parser = new XML::DOM::Parser;
  $doc = $parser->parsefile ("$db_foldername/$db_filename");
  $codebase = "";
  $imagebase = "";

  $AbsoluteFileName="$db_foldername/$db_filename";
  $AbsoluteFolderName="$db_foldername/";

  bless($self);
  return $self;
}

sub getCodebase {
  return $codebase;
}

sub getImageBase {

  my $ImageBase = ($doc->getElementsByTagName ("ImageBase"))->item (0);
  my $ImageBase_attr_url = ($ImageBase->getAttributeNode ("url"))->getValue;

  $ImageBase_attr_url =~ /\s*/;
  if ( $ImageBase_attr_url eq "" ) {
  } else {
    $imagebase = "$ImageBase_attr_url";
  }

  return $imagebase;
}

sub getAllowComment {
 
  if ( ($doc->getElementsByTagName ("AllowComment"))->item (0) == undef) {
    return "yes";
  }
  my $AllowComment = ($doc->getElementsByTagName ("AllowComment"))->item (0);
  my $isAllowComment = ($AllowComment->getAttributeNode ("status"))->getValue;
  return $isAllowComment;
}

sub getAllowUserUpload {

  if ( ($doc->getElementsByTagName ("AllowUserUpload"))->item (0) == undef) {
    return "no";
  }
  my $AllowUserUpload = ($doc->getElementsByTagName ("AllowUserUpload"))->item (0);
  my $isAllowUserUpload = ($AllowUserUpload->getAttributeNode ("status"))->getValue;
  return $isAllowUserUpload;
}


sub getAbsoluteFolderName {
  return $AbsoluteFolderName;
}

sub setImageBase {
  my $self = shift;
  $imagebase = shift;
  my $ImageBase = ($doc->getElementsByTagName ("ImageBase"))->item (0);
  $ImageBase -> setAttribute ("url", $imagebase);
  return 1;
}

sub setAllowComment {
  my $self = shift;
  $isAllowComment = shift;
  my $AllowComment = ($doc->getElementsByTagName ("AllowComment"))->item (0);
  if ( $AllowComment == null ) {
    my $AlbumDB = $doc->getDocumentElement;
    $AllowComment = $doc->createElement ("AllowComment");
    $AlbumDB->appendChild($AllowComment);
  }
  $AllowComment -> setAttribute ("status", $isAllowComment);
  return 1;
}

sub setAllowUserUpload {
  my $self = shift;
  $isAllowUserUpload = shift;
  my $AllowUserUpload = ($doc->getElementsByTagName ("AllowUserUpload"))->item (0);
  if ( $AllowUserUpload == null ) {
    my $AlbumDB = $doc->getDocumentElement;
    $AllowUserUpload = $doc->createElement ("AllowUserUpload");
    $AlbumDB->appendChild($AllowUserUpload);
  }
  $AllowUserUpload -> setAttribute ("status", $isAllowUserUpload);
  return 1;
}

# sub getImageMagickBin {
#   return $ImageMagickBin;
# }

sub addGlobalDef {

#Func: add the Album Golable definations
#Para: a reference of a hash containing {Title}, {Introduction}, {Closing}

  my $self = shift;
  my $valueref = shift;
  my %value = %{$valueref};
#  if ( $value{Title} == undef ) { $value{Title} = " "; }
#  if ( $value{Introduction} == undef ) { $value{Introduction} = " "; }
#  if ( $value{Closing} == undef ) { $value{Closing} = " "; }

  my $AlbumDB = $doc->getDocumentElement;
  my $GlobalDef = $doc->createElement ("GlobalDef");

  my $Title = $doc->createElement ("Title");
  $Title->appendChild ($doc->createTextNode(encode( $value{Title} )));

  my $Introduction = $doc->createElement ("Introduction");
  $Introduction->appendChild ($doc->createTextNode(encode( $value{Introduction} )));

  my $Closing = $doc->createElement ("Closing");
  $Closing->appendChild ($doc->createTextNode(encode( $value{Closing} )));

  $GlobalDef->appendChild ($Title);
  $GlobalDef->appendChild ($Introduction);
  $GlobalDef->appendChild ($Closing);

  $AlbumDB->appendChild ($GlobalDef);
  return 1;
}

sub editGlobalDef {

#Func: edit the Album Golable definations
#Para: a hash containing {Title}, {Introduction}, {Closing}

  my $self = shift;
  my $valueref = shift;
  my %value = %{$valueref};

  my $GlobalDef = ($doc->getElementsByTagName ("GlobalDef"))->item (0);

  if ( !($value{Title} eq "") ) { 
    my $Title = ($GlobalDef->getElementsByTagName ("Title"))->item (0);
    ($Title->getFirstChild)->setData(encode( $value{Title} ));
  }

  if ( !($value{Introduction} eq "") ) { 
    my $Introduction=($GlobalDef->getElementsByTagName("Introduction"))->item(0);
    ($Introduction->getFirstChild)->setData(encode( $value{Introduction} ));
  }

  if ( !($value{Closing} eq "") ) { 
    my $Closing = ($GlobalDef->getElementsByTagName("Closing"))->item(0);
    ($Closing->getFirstChild)->setData(encode( $value{Closing} ));
  }

  return 1;
}

sub getGlobalDef {

#Func: return a hash containing {Introduction}, {Closing}, {Title}

  my $GlobalDef = ($doc->getElementsByTagName ("GlobalDef"))->item (0);

  my $Title = ($GlobalDef->getElementsByTagName("Title"))->item(0);
  my $Title_data = decode( ($Title->getFirstChild)->getData );

  my $Introduction=($GlobalDef->getElementsByTagName("Introduction"))->item(0);
  my $Introduction_data = decode( ($Introduction->getFirstChild)->getData );

  my $Closing = ($GlobalDef->getElementsByTagName ("Closing"))->item (0);
  my $Closing_data = decode( ($Closing->getFirstChild)->getData );

#  my $ImageBase = ($GlobalDef->getElementsByTagName ("ImageBase"))->item (0);
#  my $ImageBase_attr_url = ($ImageBase->getAttributeNode ("url"))->getValue;

#  my $AllowComment = ($GlobalDef->getElementsByTagName ("AllowComment"))->item (0);
#  my $isAllowComment = ($AllowComment->getAttributeNode ("status"))->getValue;

#  my $AllowUserUpload = ($GlobalDef->getElementsByTagName ("AllowUserUpload"))->item (0);
#  my $isAllowUserUpload = ($AllowUserUpload->getAttributeNode ("status"))->getValue;

  my %result = ();
  $result{Title} = $Title_data;
  $result{Introduction} = $Introduction_data;
  $result{Closing} = $Closing_data;
  $result{ImageBase} = $$ImageBase_attr_url;
  return %result;
 
}

sub getImageNameList {

# return an array of viewable image names

  my $ImageList = $doc->getElementsByTagName ("Image");
  my $n_img = $ImageList->getLength;
  my @imagenamelist;

  for (my $i = 0; $i < $n_img; $i++) {
    my $Image = $ImageList->item ($i);
    $imagenamelist[$i] = ($Image->getAttributeNode ("filename"))->getValue; 
  }

  return @imagenamelist;
}

sub getViewableImageNameList {

# return an array of viewable image names

  my $ImageList = $doc->getElementsByTagName ("Image");
  my $n_img = $ImageList->getLength;
  my @viewableimagenamelist;

  my $j = 0;
  for (my $i = 0; $i < $n_img; $i++) {
    my $viewable = "no";
    my $Image = $ImageList->item ($i);

    if ( ($Image->getAttributeNode ("hide")) != undef ) {
      if ( (($Image->getAttributeNode ("hide"))->getValue) ne "yes" ) {
        $viewable = "yes";
      }
    } else {
      $viewable = "yes";
    }

    if ( $viewable eq "yes" ) {
      $viewableimagenamelist[$j] = ($Image->getAttributeNode ("filename"))->getValue; 
      $j = $j + 1;
    }
  }

  return @viewableimagenamelist;
}


sub getImage {
 
#Func: return an associate array containing {ThumbnailName} 
#{ImageTitle}, {ImageDescription}, {Hidden}
#Para: "Image FileName"
 
  my $self = shift;
  my $ImageName = shift;
  my %result = ();

  my $ImageList = $doc->getElementsByTagName ("Image");
  my $n_img = $ImageList->getLength;


  for (my $i = 0; $i < $n_img; $i++) {
    my $Image = $ImageList->item ($i);
    my $filename = ($Image->getAttributeNode ("filename"))->getValue;

    if ( $filename eq $ImageName ) {
      $result{ThumbnailName} = ($Image->getAttributeNode ("thumbnail"))->getValue;

      my $ImageTitle = ($Image->getElementsByTagName ("ImageTitle"))->item (0);
      $result{ImageTitle} = decode( ($ImageTitle->getFirstChild)->getData );

      my $ImageDescription = ($Image->getElementsByTagName ("ImageDescription"))-> item (0);
      $result{ImageDescription} = decode( ($ImageDescription->getFirstChild)->getData );
      if ( ($Image->getAttributeNode ("hide")) == undef ) {
        $result{Hidden} = "no";
      } else {
        $result{Hidden} = ($Image->getAttributeNode ("hide"))->getValue;
      }
      if ( ($Image->getAttributeNode ("fullimage")) == undef ) {
        $result{FullImage} = " ";
      } else {
        $result{FullImage} = ($Image->getAttributeNode ("fullimage"))->getValue;
      }
    }
  }
  return %result;
}

sub editImage {

#Para: a hash containing {ImageName}, {ThumbnailName},
#{ImageTitle}, {ImageDescription}, {Hidden}

  my $self = shift;
  my $valueref = shift;
  my %value = %{$valueref};

  my $ImageList = $doc->getElementsByTagName ("Image");
  my $n_img = $ImageList->getLength;

  for (my $i = 0; $i < $n_img; $i++) {
    my $Image = $ImageList->item ($i);
    my $filename = ($Image->getAttributeNode ("filename"))->getValue;;

    if ( $filename eq $value{ImageName} ) {

      if ( !($value{ThumbnailName} eq "") ) { 
        $Image->setAttribute ("thumbnail", $value{ThumbnailName});
      }
      if ( !($value{FullImage} eq "") ) { 
        $Image->setAttribute ("fullimage", $value{FullImage});
      }
      if ( !($value{Hidden} eq "") ) { 
        $Image->setAttribute ("hide", $value{Hidden});
      }
      if ( !($value{ImageTitle} eq "") ) { 
        my $ImageTitle = ($Image->getElementsByTagName ("ImageTitle"))-> item(0);
        ($ImageTitle->getFirstChild)->setData(encode( $value{ImageTitle} ));
      }
      if ( !($value{ImageDescription} eq "") ) { 
        my $ImageDescription = ($Image->getElementsByTagName ("ImageDescription"))-> item(0);
        ($ImageDescription->getFirstChild)->setData(encode( $value{ImageDescription} ));
      }

    }
  }
  return 1;
}

sub addImage {
 
#Para: a reference of a hash containing {ImageName}, {ThumbnailName},
#{FullImage}, {ImageTitle}, {ImageDescription}, {Hidden}

  my $self = shift;
  my $valueref = shift;
  my %value = %{$valueref};

 if ( $value{ImageName} eq "" ) { $value{ImageName} = " "; }
 if ( $value{ThumbnailName} eq "" ) { $value{ThumbnailName} = " "; }
 if ( $value{FullImage} eq "" ) { $value{FullImage} = " "; }
 if ( $value{ImageTitle} eq "" ) { $value{ImageTitle} = " "; }
 if ( $value{ImageDescription} eq "" ) { $value{ImageDescription} = " "; }
 if ( $value{Hidden} eq "" ) { $value{Hidden} = "no"; }

  my $AlbumDB = $doc->getDocumentElement;

  my $Image = $doc->createElement ("Image");
  $Image->setAttribute ("filename", $value{ImageName});
  $Image->setAttribute ("thumbnail", $value{ThumbnailName});
  $Image->setAttribute ("fullimage", $value{FullImage});
  $Image->setAttribute ("hide", $value{Hidden});

  my $ImageTitle = $doc->createElement ("ImageTitle");
  $ImageTitle->appendChild ($doc->createTextNode(encode( $value{ImageTitle} )));

  my $ImageDescription = $doc->createElement ("ImageDescription");
  $ImageDescription->appendChild ($doc->createTextNode(encode( $value{ImageDescription} )));

  my $Comments = $doc->createElement ("Comments");

  $Image->appendChild ($ImageTitle);
  $Image->appendChild ($ImageDescription);
  $Image->appendChild ($Comments);

  $AlbumDB->appendChild ($Image);
  return 1;

}

sub getImageComments {

#Func: return an array in format of [CommentNo], [User], [CommentText] ...
#CommentNo is 0,1,2,3...
#Para: "Image FileName"

  my $self = shift;
  my $ImageName = shift;
  my @result = ();

  my $ImageList = $doc->getElementsByTagName ("Image");
  my $n_img = $ImageList->getLength;

  for (my $i = 0; $i < $n_img; $i++) {
    my $Image = $ImageList->item ($i);
    my $filename = ($Image->getAttributeNode ("filename"))->getValue;

    if ( $filename eq $ImageName ) {
      my $Comments = ($Image->getElementsByTagName ("Comments"))-> item(0);
      my $CommentList = $Comments->getElementsByTagName ("Comment");
      my $n_comment = $CommentList->getLength;

      for (my $j = 0; $j < $n_comment; $j++) {
        my $Comment = $CommentList->item ($j);
        $result[$j*3] = $j;
        $result[$j*3+1] = decode( ($Comment->getAttributeNode ("name"))->getValue );
        $result[$j*3+2] = decode( ($Comment->getFirstChild)->getData );
      }
    }
  }
  return @result;
}

sub addImageComment {

#Func: Add a new comment to the image
#Para: a hash containing {ImageName}, {User}, {CommentText}

  my $self = shift;
  my $valueref = shift;
  my %value = %{$valueref};

  my $ImageList = $doc->getElementsByTagName ("Image");
  my $n_img = $ImageList->getLength;

  for (my $i = 0; $i < $n_img; $i++) {
    my $Image = $ImageList->item ($i);
    my $filename = ($Image->getAttributeNode ("filename"))->getValue;

    if ( $filename eq $value{ImageName} ) {
      my $Comments = ($Image->getElementsByTagName ("Comments"))-> item(0);

      my $Comment = $doc->createElement (Comment);
      $Comment->setAttribute("name", encode($value{User}));
      $Comment->appendChild ($doc->createTextNode(encode($value{CommentText})));

      $Comments->appendChild ($Comment);
    }
  }
  return 1;
}

sub deleteImageComment {

#Func: delete a comment. return nothing
#Para: "ImageName", "CommentNo" (From 0)

  my $self = shift;
  my $ImageName = shift;
  my $CommentNo = shift;

  my $ImageList = $doc->getElementsByTagName ("Image");
  my $n_img = $ImageList->getLength;

  for (my $i = 0; $i < $n_img; $i++) {
    my $Image = $ImageList->item ($i);
    my $filename = ($Image->getAttributeNode ("filename"))->getValue;
    if ( $filename eq $ImageName ) {
      my $Comments = ($Image->getElementsByTagName ("Comments"))-> item(0);
      my $CommentList = $Comments->getElementsByTagName ("Comment");
      my $Comment = $CommentList->item ($CommentNo);
      $Comments -> removeChild ($Comment);
    }
  }
  return 1;
}

sub exportDoc {

#Func: Export the doc object to db file. return nothing

  while (-e "$AbsoluteFolderName/lock.file") { }

  system "echo this_is_a_lock > $AbsoluteFolderName/lock.file";
  $doc->printToFile ("$AbsoluteFileName");
  breakLines ("$AbsoluteFileName");
  system "rm $AbsoluteFolderName/lock.file";
  return 1;

}

sub DESTROY {

  exportDoc ();
  return 0;

}

sub breakLines {

#This one is a private function. It cannot be called from outside.

  my $filename = shift;

  my $element;
  open (DOCFILE, "<$filename");
  my @oldfile = <DOCFILE>;
  close (DOCFILE);

  open (DOCFILE, ">$filename");
  foreach $element ( @oldfile ) {
    $element =~ s/><Image\s+/>\n<Image /g;
    print DOCFILE "$element";
  }
  close (DOCFILE);

  return 1;
}

sub encode {

#private function to encode Values bigger than ASCII 126 and put the
#ASCII value between a pair of &!xxx!&

  my @oldarray;
  my $oldstring;
  my @newarray;
  my $newstring;

  $oldstring = shift;
  if ( $oldstring eq "" ) {
    $oldstring = " ";
  }
  @oldarray = split //, $oldstring;

  for (my $i = 0; $i <= $#oldarray ; $i=$i+1) {
    if ( ord($oldarray[$i]) > 128 ) {
      my $newchar = ord($oldarray[$i]);
      my @newchars = split //, "#!$newchar!#";
      for (my $j = 0; $j <= @newchars; $j = $j + 1) {
        push @newarray, $newchars[$j];
      }
    } else {
      push @newarray, $oldarray[$i];
    }
  }

  $newstring = join("", @newarray);
  return $newstring;
}

sub decode {

#private function to decode the &!xxx!& in string for ASCII values > 126
#and put them back in the string

  my @oldarray;
  my $oldstring;
  my @newarray;
  my $newstring;
  my @charvalue;

  $oldstring = shift;
  @oldarray = split //, $oldstring;
  for (my $i = 0; $i <= $#oldarray ; $i=$i+1) {
    if ($oldarray[$i] eq '#' ) {
      $i = $i + 1;
      if ($oldarray[$i] eq '!' ) {
        $i = $i + 1;
        my $j = 0;
        while ( $oldarray[$i] =~ /\d/ ) {
          $charvalue[$j] = $oldarray[$i];
          $j = $j +1 ;
          $i = $i +1 ;
        }
      }
      $i = $i + 1;
      push @newarray, chr( join("", @charvalue) );
    }  else {
      push @newarray, $oldarray[$i];
    }
  }
  $newstring = join("", @newarray);
  return $newstring;
}

sub isAllowed {

# check if the string contains illegal HTML tags. 
# return yes or no

  my $class = shift;
  my $dirtystring = shift;

#We need to loop though all the tags, so we use "while" here

  while ( $dirtystring =~ m/<(\w+)/g ) {
    my $matchedtag = $1;
    my $islegal = "no";
    $matchedtag = uc $matchedtag;
    foreach $legaltag (@allowedtags) {
      if ( $matchedtag eq $legaltag ) {
        $islegal = "yes";
      }
    }
    if ( $islegal eq "no" ) {
      return "no";
    }
  }
  return "yes";
}

sub getFullImagesFromFolder {

  my @allimages = ();
  my @exts = ("jpg", "JPG", "jpeg", "JPEG", "gif", "GIF", "png", "PNG");

  my $currentPath = getcwd;
  chdir "$AbsoluteFolderName";
  my $i = 0;
  foreach (@exts) {
    my $ext = $_;
    my @imgfiles = <*.$ext>;
    foreach (@imgfiles) {
      $img_filename = $_;
      if ($img_filename =~ m/^TN-/ || -e "TN-$img_filename" ) {
      } else {
        $allimages[$i] = $img_filename;
        $i = $i + 1;
      }
    }
  }
  chdir "$currentPath";
  return @allimages;
}

sub setThumbnailLock {

  my $currentPath = getcwd;
  chdir "$AbsoluteFolderName";
  system ("echo thumbnail-in-progress > thumbnail-in-progress");
  chdir "$currentPath";
}

sub releaseThumbnailLock {
  my $currentPath = getcwd;
  chdir "$AbsoluteFolderName";
  system ("rm thumbnail-in-progress");
  chdir "$currentPath";
}

sub getThumbnailLock {
  my $result = "no";
  my $currentPath = getcwd;
  chdir "$AbsoluteFolderName";
  if ( -e "thumbnail-in-progress" ) {
    $result = "yes";
  } else {
    $result = "no";
  }
  chdir "$currentPath";
  return $result;
}

sub resize {

# generate resized images (thumbnails) in the AbsoluteFolderName
# and return the image list

  my $class = shift;
  my $img_filename = shift;
  my $resizeop = shift;
  my $prefix = shift;

  $img_thumbnail = "$prefix-$img_filename";
  if ( $resizeop == undef || $resizeop eq "" ) {
    $resizeop = "25%x25%";
  }

  my $currentPath = getcwd;
  chdir "$AbsoluteFolderName";

#sometimes the mogrify fails to update the thumbnail image after
#processing. Here is to test if this ever happens and redo it when
#it did.

  my $filediff = 0;
  while ( $filediff == 0 ) {
    system "cp $img_filename $img_thumbnail";
    system "$ImageMagickBin/mogrify -geometry $resizeop! $img_thumbnail";
    my @fileinfo = stat($img_filename);
    my @thumbinfo = stat($img_thumbnail);
    $filediff = $fileinfo[7] - $thumbinfo[7];
  }
  chdir "$currentPath";
}

return 1;
