#!/usr/local/bin/perl

## You have to call this program from the cgi-bin/admin directory.
## the first parameter is the absolute path to the root folder
## for the albums that you want to generate HTML pages. 

## HOW IT WORKS
## 1. Call this program from cgi-bin/admin with $rootdir to the album
## 2. It remembers its config.xml path in $initdir/../config.xml
## 3. It generates all .xml indexes in the album $rootdir recursively
## 4. Loop through all .xml indexes and call makeHTML using absolute
##    paths and index filenames. IGNORE backup.xml and test.xml
## 5. makeHTML will then print out a URL to that album using relative
##    path from $rootdir
## 6. make HTML enters a folder, read off the index and make HTML 
##    directories and files.
## 7. Repeat No.4

push(@INC, "..");
use CGI;
use Cwd;
require("DBAccess.pl");


$rootdir = $ARGV[0];
$initdir = getcwd ();

@list =();
listdirectory($rootdir);
for $loopindex (0..$#list) {
  my $dirname = $list[$loopindex]{dirname};
  my $filename = $list[$loopindex]{filename};
  if ( $filename ne "backup.xml" && $filename ne "test.xml" ) {
    makeHTML($dirname, $filename);
  }
}

sub makeHTML {

## dirname is the absolute path to the folder
## and filename is the filename for the index

  my $dirname = shift;
  my $filename = shift;
  $filename =~ s/\.xml//g;

## for test
## print "$dirname >> $filename";

  my $db = DBAccess->newLocal("$initdir/../config.xml", "$dirname", "$filename.xml");
  my @imagenames = $db -> getViewableImageNameList ();
  my %globaldata = $db -> getGlobalDef ();

## print out the current working album
  my $shortdirname = $dirname;
  $shortdirname =~ s|^$rootdir/||;
  print "<p><a href=\"$shortdirname/HTML/$filename/index.html\"> $globaldata{Title} </a> \n";

  chdir "$dirname" 
   or die "cannot change to $dirname";

#delete first!
  if ( !(-e HTML) ) {
    mkdir "HTML", 0744;
  }
  chdir "HTML";
  if ( !(-e $filename) ) {
    mkdir "$filename", 0744;
  }
  chdir "$filename";

  open (INDEX_FILE, ">index.html") or
       die ("cannot open index.html");

  print INDEX_FILE "<html><head><title>Image viewer</title></head>\n";
  print INDEX_FILE "<body bgcolor=\"#FFFFFF\">\n";
  print INDEX_FILE "<table width=100%><tr><td align=right>";
  print INDEX_FILE "<a href=\"$imagenames[0].slide.html\">SlideShow</a>";
  print INDEX_FILE "</td></tr></table><center> \n";
  print INDEX_FILE "<p><h1><font face=\"Arial,Helvetica\">$globaldata{Title} </font></h1>\n";
  print INDEX_FILE "<p><font face=\"Arial,Helvetica\">$globaldata{Introduction}</font>\n";

  my $counter = 0;
  print INDEX_FILE "<table border><tr> \n";
  for ( my $i = 0; $i <= $#imagenames; $i = $i + 1 ) {
    my %imagedata = $db -> getImage($imagenames[$i]);
    print INDEX_FILE "<td><a href=\"$imagenames[$i].show.html\"><img src=\"../../$imagedata{ThumbnailName}\"/></a>";
    print INDEX_FILE "<p><font face=\"Arial,Helvetica\" size=2>$imagedata{ImageTitle}</font></td> \n";
    $counter = $counter + 1;
    if ( $counter == 4 ) {
      $counter = 0;
      print INDEX_FILE "</tr><tr> \n";
    }
  }
  print INDEX_FILE "</tr></table> \n";
  print INDEX_FILE "<p><font face=\"Arial,Helvetica\" size=2>$globaldata{Closing}</font>\n";
  print INDEX_FILE "</center></body></html>\n";

  close (INDEX_FILE);

  for ( my $i = 0; $i <= $#imagenames; $i = $i + 1 ) {

    my %imageinfo = $db -> getImage( $imagenames[$i] );
    my @imagecomm = $db -> getImageComments( $imagenames[$i] );

    open (SHOW_FILE, ">$imagenames[$i].show.html") or
         die ("cannot open $imagenames[$i].show.html");

    my $prev_image = "";
    my $next_image = "";
    for ( my $j = 0; $j <= $#imagenames; $j = $j + 1 ) {
      if ( $imagenames[$i] eq $imagenames[$j] ) {
        if ( $j == 0 ) {
        } else {
          $prev_image = $imagenames[$j-1];
        }
        if ( $j == $#imagenames ) {
        } else {
          $next_image = $imagenames[$j+1];
        }
      }
    }
    print SHOW_FILE "<html><head><title>Image viewer</title></head>\n";
    print SHOW_FILE "<body bgcolor=\"#FFFFFF\">\n";
    print SHOW_FILE "<table width=100%><tr><td align=left>";
    if ( $prev_image ne "" ) {
      print SHOW_FILE "<a href=\"$prev_image.show.html\">Prev</a> || ";
    }
    if ( $next_image ne "" ) {
      print SHOW_FILE "<a href=\"$next_image.show.html\">Next</a> ";
    }
    print SHOW_FILE "</td></tr><tr><td align=right>";
    print SHOW_FILE "<a href=\"index.html\">Index</a>";
    print SHOW_FILE "</td></tr></table><center> \n";
    print SHOW_FILE "<p><h1>$imageinfo{ImageTitle}</h1> \n";
    print SHOW_FILE "<p>$imageinfo{ImageDescription} </center>\n";

    my $comments_no = ($#imagecomm + 1)/3;
    print SHOW_FILE "<p>It has <font color=red>$comments_no</font> comments\n";

    print SHOW_FILE "<center> \n";
    if (" " ne $imageinfo{FullImage}) {
      print SHOW_FILE "<p><a href=\"$imagebase/$imageinfo{FullImage}\"> ";
      print SHOW_FILE "High Quality Image </a>";
    }
    print SHOW_FILE "<p><img src=\"../../$imagenames[$i]\"></center>";

    if ($#imagecomm > 0) {
      print SHOW_FILE "<p>Comments \n";
    }
    print SHOW_FILE "<table border> \n";
    for (my $j = 0; $j < ($#imagecomm+1)/3; $j++) {
      print SHOW_FILE "<tr><td><b>$imagecomm[$j*3+1]</b></td> \n";
      print SHOW_FILE "<td>$imagecomm[$j*3+2]</td></tr> \n";
    }
    print SHOW_FILE "</table> \n";
    print SHOW_FILE "</body></html> \n";

    close (SHOW_FILE);


    open (SLIDE_FILE, ">$imagenames[$i].slide.html") or
         die ("cannot open $imagenames[$i].slide.html");

    if ( $i == $#imagenames ) {
      print SLIDE_FILE "<html><head><meta http-equiv=\"REFRESH\" content=\"0; URL=index.html\"></head> \n";
    } else {
      print SLIDE_FILE "<html><head><meta http-equiv=\"REFRESH\" content=\"5; URL=$imagenames[$i+1].slide.html\"></head> \n";
    }
    print SLIDE_FILE "<html><head><title>Image viewer</title></head>\n";
    print SLIDE_FILE "<body bgcolor=\"#FFFFFF\">\n";
    print SLIDE_FILE "<table width=100%><tr><td align=left>";
    print SLIDE_FILE "</td></tr><tr><td align=right>";
    print SLIDE_FILE "<a href=\"index.html\">Stop</a>";
    print SLIDE_FILE "</td></tr></table><center> \n";
    print SLIDE_FILE "<p><h1>$imageinfo{ImageTitle}</h1> \n";
    print SLIDE_FILE "<p>$imageinfo{ImageDescription} </center>\n";

    my $comments_no = ($#imagecomm + 1)/3;
    print SLIDE_FILE "<p>It has <font color=red>$comments_no</font> comments\n";

    print SLIDE_FILE "<center><p><img src=\"../../$imagenames[$i]\"></center>";
    if ($#imagecomm > 0) {
      print SLIDE_FILE "<p>Comments \n";
    }
    print SLIDE_FILE "<table border> \n";
    for (my $j = 0; $j < ($#imagecomm+1)/3; $j++) {
      print SLIDE_FILE "<tr><td><b>$imagecomm[$j*3+1]</b></td> \n";
      print SLIDE_FILE "<td>$imagecomm[$j*3+2]</td></tr> \n";
    }
    print SLIDE_FILE "</table> \n";
    print SLIDE_FILE "</body></html> \n";

    close (SLIDE_FILE);
  }
}

sub listdirectory {
  local($dir);
  local(@lines);
  local($fileitem);
  local($lvl_counter);
  local($list_length);

  $dir = shift;
  if(opendir (DIR, $dir)) {
    @lines = readdir (DIR);
    closedir (DIR);
    $lvl_counter = 2;
    $list_length = ( scalar @lines );
    while ($lvl_counter < $list_length) {
      $fileitem = $dir."/".$lines[$lvl_counter];
      if( -d $fileitem) {
         listdirectory($fileitem);
      } else {
        if ( $lines[$lvl_counter] =~ m/\.xml/ ) {
          push @list, {dirname => "$dir", filename => "$lines[$lvl_counter]"};
        }
      }
      $lvl_counter++;
    }
  }
}

